/*
###########################################################################
# 
# Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
# rights reserved.
# 
# Developed  by Ali Azarbayejani and Alex Pentland,  and Martin  Friedmann
# at the Media Laboratory, MIT, Cambridge, Massachusetts.
# 
# Permission to use, copy,  or modify this software and  its documentation
# for  educational  and  research purposes only and without fee  is hereby
# granted, provided  that this copyright notice and the original authors's
# names appear  on all copies and supporting documentation.  If individual
# files are  separated from  this  distribution directory  structure, this
# copyright notice must be included.  For any other uses of this software,
# in original or  modified form, including but not limited to distribution
# in whole or in  part, specific  prior permission  must be  obtained from
# MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
#   basis  of  a commercial  software  or  hardware product  without first
# obtaining appropriate licenses  from MIT.  MIT. makes no representations
# about the suitability of this  software for any purpose.  It is provided
# "as is" without express or implied warranty.
# 
###########################################################################

  ekf.h
  8 December 1992 (originated)

  AUTHOR:
    Ali Azarbayejani

  CONTENTS:
    ekf_type

  DESCRIPTION:
    Based on:
    kalman.h -- Structure and macros for the Extended Kalman Filter abstraction
                Author:	Martin R. Friedmann
   		Date:	Mon Mar 25 12:41:08 EST 1991

  REFERENCES:
    Brown, Robert G., << Introduction to Random Signal Analysis and Kalman
        Filtering >>, John Wiley & Sons, New York, 1983

    Gelb, Arthur (Editor), << Applied Optimal Estimation >>, MIT Press,
       Cambridge, MA, 1974

    Friedmann, Martin, Alex Pentland, and Thad Starner, "Device
       Synchronization using an Optimal Linear Filter", Proceedings of the
       ACM Symposium on Interactive 3D Graphics, Cambridge, MA, Vol 26, No
       2, March 1992

    Azarbayejani, Ali and Alex Pentland, "Recursive Estimation of Motion,
       Structure, and Focal Length", IEEE PAMI, June 1995

*/

#include "matrix/matrix.h"
#include "matrix/sub_matrix.h"

#ifndef _EKF_H_
#define _EKF_H_

typedef struct _ekf_type {
  char *name;
  
  int N_max, P_max;		/* MAXIMUM State and Signal vector sizes */
  int n, p;			/* State and Signal vector sizes */
  
  FLOATTYPE *Y;			/* actual signal vector */
  FLOATTYPE *Y_p;		/* predicted signal vector */
  FLOATTYPE *dY;		/* increm. signal vec. (sig - predicted) */
  FLOATTYPE *X_p;		/* State Vector Prediction */
  FLOATTYPE *dX;		/* increm. state vec (est - pred) */
  FLOATTYPE *X_e;		/* Optimal State Estimate */
  
  flex_matrix_type *I;		/* NxN Identity Matrix */
  flex_matrix_type *P_p;	/* NxN Error Covariance Prediction */
  flex_matrix_type *P_e;	/* NxN Error Covariance Estimate */
  flex_matrix_type *H;		/* PxN Linearized Signal Vector  */
  flex_matrix_type *Q;		/* NxN State Error Covariance Matrix */
  flex_matrix_type *R;		/* PxP Signal Error Covariance Matrix */
  flex_matrix_type *K;		/* NxP Kalman Gain Matrix */
  flex_matrix_type *Phi;	/* State Transition Matrix */
  flex_matrix_type *TPxP;	/* temporary matrix... */
  flex_matrix_type *TPxN;	/* temporary matrix... */
  flex_matrix_type *TNxP;	/* temporary matrix... */
  flex_matrix_type *TNxN;	/* temporary matrix... */
  
  void (*initialize)();		/* init xp, Pp, [Q, R, H, Phi]  */
  void (*pre_process)();		/* before each step (opt.)	*/
  void (*linearize_signal_model)(); 	/* NULL: H(t)=H(t-1)		*/
  void (*predict_signal)();		/* NULL: Yp(t)=H(t)Xp(t)        */
  void (*estimate_state)();		/* NULL: Xe(t)=Xp(t)+K(t)dY(t)  */
  void (*make_state_transition)();	/* NULL: Phi(t)=Phi(t-1)        */
  void (*predict_state)();		/* NULL: Xp(t+1)=Phi(t)Xe(t)    */
  void (*post_process)();		/* after each step (opt.)	*/
} ekf_type;

extern void 	ekf_print();
extern int	ekf_size_in_bytes();
extern ekf_type *ekf_make_filter();
extern void 	ekf_clear_state();
extern void	ekf_resize_filter();
extern void 	ekf_filter_step();

#endif /* _EKF_H_ */
