/*
###########################################################################
# 
# Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
# rights reserved.
# 
# Developed  by Ali Azarbayejani and Alex Pentland,  and Martin  Friedmann
# at the Media Laboratory, MIT, Cambridge, Massachusetts.
# 
# Permission to use, copy,  or modify this software and  its documentation
# for  educational  and  research purposes only and without fee  is hereby
# granted, provided  that this copyright notice and the original authors's
# names appear  on all copies and supporting documentation.  If individual
# files are  separated from  this  distribution directory  structure, this
# copyright notice must be included.  For any other uses of this software,
# in original or  modified form, including but not limited to distribution
# in whole or in  part, specific  prior permission  must be  obtained from
# MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
#   basis  of  a commercial  software  or  hardware product  without first
# obtaining appropriate licenses  from MIT.  MIT. makes no representations
# about the suitability of this  software for any purpose.  It is provided
# "as is" without express or implied warranty.
# 
###########################################################################

  matrix_print.c
  Mon Mar 11 03:49:39 EST 1991

  AUTHOR:
    Martin Friedmann

  CONTENTS:
    vector_print_nlf()
    vector_print()
    matrix_print()
    matrix_print_lu()
    matrix_print_l()
    matrix_print_u()
    vector_print_non_zeros()
    matrix_print_non_zeros()
    vector_print_bool()
    matrix_print_bool()
    matrix_print_blurb()
    vector_print_matlab()
    matrix_print_matlab()
    matrix_print_system()
    vector_print_mathematica_nlf()
    vector_print_mathematica()
    matrix_print_mathematica()

  DESCRIPTION:
    Routines for printing vectors and matrices.

*/

#include "matrix.h"

/* no line feed */
void vector_print_nlf( stream, v, len )
register FILE *stream;
register FLOATTYPE *v;
int len;
{
    register i;

    fprintf(stream, "(");
    for(i=0; i<len; i++)
	fprintf(stream, "%9.6g ", (double)*v++);
    
    fprintf(stream, ")");
}

/****************************************************************************
 * matrix_print & vector_print: Prints matrices and vectors of given width &
 * 	height or length.
 */

void vector_print( stream, v, len )
register FILE *stream;
register FLOATTYPE *v;
int len;
{
    vector_print_nlf( stream, v, len );
    fprintf(stream, "\n");
}

void matrix_print( stream, m )
register FILE *stream;
matrix_type *m;
{
    register int i, w = m->w;
    register FLOATTYPE *mm1 = m->m;

    fprintf(stream, "(%s\n", m->name);
    for (i=0; i<m->h; i++) {
	vector_print(stream, &MREF(m,i,0), m->w);
    }
    fprintf(stream, ")\n");
}

void matrix_print_lu( stream, m )
register FILE *stream;
matrix_type *m;
{
    register int i;

    if (m->row_order == NULL) 
	matrix_print( stream, m );
    else {
	fprintf(stream, "(%s\n", m->name);
	for (i=0; i<m->h; i++) {
	    vector_print(stream, &MREF(m,m->row_order[i],0), m->w);
	}
	fprintf(stream, ")\n");
    }
}

void matrix_print_l( stream, m )
register FILE *stream;
matrix_type *m;
{
    register int j, i, w = m->w;

    fprintf(stream, "(Lower %s\n", m->name);
    for (i=0; i<m->h; i++) {
	fprintf(stream, "(");
	for (j=0; j<m->w; j++) {
	    if ( i > j )
		fprintf(stream, "%g ", MREF(m,m->row_order[i],j));
	    else if (i == j)
		fprintf(stream, "%g ", 1.0);
	    else
		fprintf(stream, "%g ", 0.0);
	}
	fprintf(stream, ")\n");
    }
    fprintf(stream, ")\n");
}

void matrix_print_u( stream, m )
register FILE *stream;
matrix_type *m;
{
    register int j, i, w = m->w;

    fprintf(stream, "(Upper %s\n", m->name);
    for (i=0; i < m->h; i++) {
	fprintf(stream, "(");
	for (j=0; j < m->w; j++) {
	    if ( i <= j )
		fprintf(stream, "%g ", MREF(m,m->row_order[i],j));
	    else
		fprintf(stream, "%g ", 0.0);
	}
	fprintf(stream, ")\n");
    }
    fprintf(stream, ")\n");
}


/****************************************************************************
 * matrix_print_non_zeros & vector_print_non_zeros:
 *
 * 	Prints matrices and vectors of given width & height or length, but
 * 	only prints non-zero entries as an X.  Used to show non-zero entries
 * 	and effects of fill-in on band matrices.
 */
void vector_print_non_zeros( stream, v, len )
register FILE *stream;
register FLOATTYPE *v;
int len;
{
    register i;

    fprintf(stream, "(");
    for (i=0; i<len-1; i++)
	fprintf(stream, "%c", (*v++ == 0.0)?' ':'x');
    
    fprintf(stream, "%c)\n", (*v == 0.0)?' ':'x');
}

void matrix_print_non_zeros( stream, m )
register FILE *stream;
matrix_type *m;
{
    register int i, w = m->w;

    fprintf(stream, "(%s\n", m->name);
    for (i=0; i<m->h; i++) {
	vector_print_non_zeros(stream, &MREF(m,i,0), m->w);
    }
    fprintf(stream, ")\n");
}


void vector_print_bool( stream, v, len, bool_func )
register FILE *stream;
register FLOATTYPE *v;
int len;
int (*bool_func)();
{
    register i;

    fprintf(stream, "(");
    for (i=0; i<len-1; i++)
	fprintf(stream, "%c", (*bool_func)(*v++)?' ':'x');
    
    fprintf(stream, "%c)\n", (*bool_func)(*v)?' ':'x');
}

void matrix_print_bool( stream, m, bool_func )
register FILE *stream;
matrix_type *m;
int (*bool_func)();
{
    register int i, w = m->w;

    fprintf(stream, "(%s\n", m->name);
    for (i=0; i<m->h; i++) {
	vector_print_bool(stream, &MREF(m,i,0), m->w, bool_func);
    }
    fprintf(stream, ")\n");
}

void matrix_print_blurb( stream, m )
register FILE *stream;
matrix_type *m;
{
    fprintf(stream, "%dx%d (HxW) Matrix '%s' at %x with data at %x\n",
	    m->h, m->w, m->name, m, m->m );
}

void vector_print_matlab( stream, v, len )
register FILE *stream;
register FLOATTYPE *v;
int len;
{
    register i;

    for (i=0; i<len-1; i++)
	fprintf(stream, "%g ", (double)*v++);
    
    fprintf(stream, "%g;\n", *v);
}

void matrix_print_matlab( stream, m, symbol )
register FILE *stream;
matrix_type *m;
{
    register int i;

    fprintf(stream, "%s = [\n", symbol);
    for (i=0; i<m->h; i++) {
	vector_print_matlab(stream, &MREF(m,i,0), m->w);
    }
    fprintf(stream, "];\n");
}

void matrix_print_system( stream, m, v )
register FILE *stream;
register FLOATTYPE *v;
matrix_type *m;
{
    register int i;
    
    fprintf(stream, "(%s\n", m->name);
    for (i=0; i<m->h; i++) {
	vector_print_nlf(stream, &MREF(m,i,0), m->w);
	vector_print(stream, v++, 1);
    }
    fprintf(stream, ")\n");
}

void vector_print_mathematica_nlf( stream, v, len )
register FILE *stream;
register FLOATTYPE *v;
int len;
{
    register int i;

    fprintf(stream, "{");
    for (i=0; i<len-1; i++)
	fprintf(stream, "%9.9g, ", (double)*v++);
    fprintf(stream, "%9.9g }", (double)*v);
}

vector_print_mathematica( stream, v, len, symbol )
register FILE *stream;
register FLOATTYPE *v;
int len;
char *symbol;
{
    fprintf(stream, "%s = ", symbol);
    vector_print_mathematica_nlf(stream, v, len);
    fprintf(stream, ";\n", symbol);
}

void matrix_print_mathematica( stream, m, symbol )
register FILE *stream;
matrix_type *m;
char *symbol;
{
    register int i;

    fprintf(stream, "%s = {", symbol);
    for (i=0; i<m->h; i++) {
	vector_print_mathematica_nlf(stream, &MREF(m,i,0), m->w);
	if (i != m->h-1)
	    fprintf(stream, ",\n" );
    }
    fprintf(stream, "};\n");
}
