/*
###########################################################################
# 
# Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
# rights reserved.
# 
# Developed  by Ali Azarbayejani and Alex Pentland,  and Martin  Friedmann
# at the Media Laboratory, MIT, Cambridge, Massachusetts.
# 
# Permission to use, copy,  or modify this software and  its documentation
# for  educational  and  research purposes only and without fee  is hereby
# granted, provided  that this copyright notice and the original authors's
# names appear  on all copies and supporting documentation.  If individual
# files are  separated from  this  distribution directory  structure, this
# copyright notice must be included.  For any other uses of this software,
# in original or  modified form, including but not limited to distribution
# in whole or in  part, specific  prior permission  must be  obtained from
# MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
#   basis  of  a commercial  software  or  hardware product  without first
# obtaining appropriate licenses  from MIT.  MIT. makes no representations
# about the suitability of this  software for any purpose.  It is provided
# "as is" without express or implied warranty.
# 
###########################################################################

  sub_matrix.h
  13 December 1992

  AUTHOR:
    Ali J. Azarbayejani

  CONTENTS:
    sub_matrix_type
    flex_matrix_type

  DESCRIPTION:
    A sub_matrix_type has rows which are not necessarily contiguous
    in memory.  A sub_matrix_type, can, e.g., be imbedded in a larger
    matrix.

*/

#include "matrix.h"

#ifndef _SUB_MATRIX_H_
#define _SUB_MATRIX_H_

typedef struct {
  matrix_type	*M;	/* host matrix */
  char		*name;	/* sub_mat name */
  int		w;	/* sub_mat width */	/* [0, M.w] */
  int		h;	/* sub_mat height */
  int		line;	/* == M.w */
  FLOATTYPE	*m;	/* ptr to first element of sub_matrix */
} sub_matrix_type;

typedef sub_matrix_type flex_matrix_type;

#define SMREF(sm, y, x) AREF(sm->m, y, x, sm->line)

extern void 		fill_flex_matrix_type_given_memory();
extern flex_matrix_type *make_flex_matrix_type();
extern void		fill_sub_matrix_given_matrix();
extern sub_matrix_type 	*make_sub_matrix_type();
extern void		copy_sub_matrix_type();
extern matrix_type	*copy_sub_matrix_to_matrix();
extern void		copy_matrix_to_sub_matrix();
extern void 		sub_matrix_identity();
extern void		sub_matrix_zero();
extern void		sub_matrix_reframe();

extern void 		sub_matrix_multiply();
extern void 		sub_matrix_transpose_multiply();
extern void 		sub_matrix_multiply_transpose();
extern void 		sub_matrix_transpose();
extern void 		sub_matrix_add();
extern void 		sub_matrix_subtract();
extern void 		sub_matrix_scale();
extern void 		sub_matrix_invert();

extern void 		sub_matrix_print();
extern void 		sub_matrix_print_blurb();
extern void vector_print_non_zeros();
extern void sub_matrix_print_non_zeros();
extern void vector_print_bool();
extern void sub_matrix_print_bool();

#endif /* _SUB_MATRIX_H_ */
