/*
###########################################################################
# 
# Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
# rights reserved.
# 
# Developed  by Ali Azarbayejani and Alex Pentland,  and Martin  Friedmann
# at the Media Laboratory, MIT, Cambridge, Massachusetts.
# 
# Permission to use, copy,  or modify this software and  its documentation
# for  educational  and  research purposes only and without fee  is hereby
# granted, provided  that this copyright notice and the original authors's
# names appear  on all copies and supporting documentation.  If individual
# files are  separated from  this  distribution directory  structure, this
# copyright notice must be included.  For any other uses of this software,
# in original or  modified form, including but not limited to distribution
# in whole or in  part, specific  prior permission  must be  obtained from
# MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
#   basis  of  a commercial  software  or  hardware product  without first
# obtaining appropriate licenses  from MIT.  MIT. makes no representations
# about the suitability of this  software for any purpose.  It is provided
# "as is" without express or implied warranty.
# 
###########################################################################

  dattofts.c
  14 March 1994

  CONTENTS:

  DESCRIPTION:
*/

#include <stdio.h>
#include <stdlib.h>

#define Min( a, b )	(((a)<(b)) ? (a) : (b))
#define Max( a, b )	(((a)>(b)) ? (a) : (b))
#define Limit( x, a, b ) Min( Max( x, a ), b )

struct {
  char	*progname;
  int	verbose;
  int   nfe;
  int   w, h;
  int	pts;
  float	xx, xy, yx, yy, x0, y0;
}

params = {
  "", 0,
  22,
  255, 246,
  0,
  1., 0., 0., 1., 0., 0.
};

static void
  alert( char *s )
{
  fprintf( stderr, "%s: %s\n", params.progname, s );
}

static void
  error( char *s )
{
  alert( s );
  exit( -1 );
}

static void
  usage( int argc, char **argv )
{
  static char *flags = "[-v]";
  static char *io    = "< (stream) > (stream)";
  static char *help  = "";

  fprintf( stderr, "usage: %s %s\n\t%s\n%s", argv[0], flags, io, help );
  exit(-1);
}

static void
  parse_command_line( int argc, char **argv )
{
  int		i;

  params.progname = argv[0];

  for(i=1; i<argc; i++) {
    if( !strcmp(argv[i], "-v") )
      params.verbose = 1;
    else if( !strcmp(argv[i], "-w") )
      params.w = atoi( argv[++i] );
    else if( !strcmp(argv[i], "-h") )
      params.h = atoi( argv[++i] );
    else if( !strcmp(argv[i], "-nfe") )
      params.nfe = atoi( argv[++i] );
    else if( !strcmp(argv[i], "-pts") )
      params.pts = 1;
    else if( !strcmp(argv[i], "-x0") )
      params.x0 = atof( argv[ ++i ] );
    else if( !strcmp(argv[i], "-y0") )
      params.y0 = atof( argv[ ++i ] );
    else if( !strcmp(argv[i], "-A") ) {
      params.xx = atof( argv[ ++i ] );
      params.xy = atof( argv[ ++i ] );
      params.yx = atof( argv[ ++i ] );
      params.yy = atof( argv[ ++i ] );
    }
    else
      usage( argc, argv );
  }
}

#define NEW_X( x, y, unit ) ( (params.xx*(x) + params.xy*(y) + params.x0) / (unit) )
#define NEW_Y( x, y, unit ) ( (params.yx*(x) + params.yy*(y) + params.y0) / (unit) )

int
  scan_frame( int nfe, int w, int h )
{
  static int first_frame = 1;
  int rv = EOF;
  float x, y;
  int count;
  float unit = Max( w, h );

  if( params.pts ) {
    rv = scanf( "%d", &count );
    if( EOF == rv ) return EOF;
    if( count != nfe )
      error( "pts count not equal to specified number features" );
  }
  rv = scanf( "%f%f", &x, &y );
  if( EOF == rv )
    return EOF;
  printf( "KILL 0\n" );

  if( first_frame ) {
    printf( "OLD 0\n" );
    printf( "NEW %d\n", nfe );
    for( count = 0; count < nfe; count++ ) {
      if( count ) rv = scanf( "%f%f", &x, &y );
      if( 2 != rv )
        alert( "ERROR, ERROR: rv not 2" );
      printf( "  %d %.10g %.10g\n", count, NEW_X(x,y,unit), NEW_Y(x,y,unit) );
    }
    printf( "\n" );
  }
  else {
    printf( "OLD %d\n", nfe );
    for( count = 0; count < nfe; count++ ) {
      if( count ) rv = scanf( "%f%f", &x, &y );
      if( 2 != rv )
        alert( "ERROR, ERROR: rv not 2" );
      printf( "  %d %.10g %.10g %.10g %.10g\n",
	     count, NEW_X(x,y,unit), NEW_Y(x,y,unit), 0., 0. );
    }
    printf( "NEW 0\n" );
    printf( "\n" );
  }

  first_frame = 0;
  return rv;
}

main( int argc, char **argv )
{
  /* Main variables */

  /* First command */
  parse_command_line( argc, argv );

  while( EOF != scan_frame( params.nfe, params.w, params.h ) );
}
