/*
###########################################################################
# 
# Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
# rights reserved.
# 
# Developed  by Ali Azarbayejani and Alex Pentland,  and Martin  Friedmann
# at the Media Laboratory, MIT, Cambridge, Massachusetts.
# 
# Permission to use, copy,  or modify this software and  its documentation
# for  educational  and  research purposes only and without fee  is hereby
# granted, provided  that this copyright notice and the original authors's
# names appear  on all copies and supporting documentation.  If individual
# files are  separated from  this  distribution directory  structure, this
# copyright notice must be included.  For any other uses of this software,
# in original or  modified form, including but not limited to distribution
# in whole or in  part, specific  prior permission  must be  obtained from
# MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
#   basis  of  a commercial  software  or  hardware product  without first
# obtaining appropriate licenses  from MIT.  MIT. makes no representations
# about the suitability of this  software for any purpose.  It is provided
# "as is" without express or implied warranty.
# 
###########################################################################

  files.h
  2 June 1993

  AUTHOR:
    Ali J. Azarbayejani 

  CONTENTS:
    typedef fts_type
    typedef est_type
    typedef sta_type

  DESCRIPTION:
    fts_type: features
    est_type: estimate of state (dynamic parameters)
    sta_type: static parameters (structure, camera)
  
*/

#ifndef _FILES_H_
#define _FILES_H_

#define FILES_CLOSED		0
#define FILES_OPEN_READ		1
#define	FILES_OPEN_WRITE	2

#define	FILES_LIST_INIT		100
#define	FILES_LIST_INC		25

               /********************************\
----------------------------------------------------------------
  (.fts) files
----------------------------------------------------------------
               \********************************/

typedef int kill_type;
typedef struct { int id; float x, y, Vx, Vy; } old_type;
typedef struct { int id; float x, y; } new_type;

typedef struct { int kind, id; float x, y, Vx, Vy; } generic_type;
#define	FILES_KILL_KIND 0
#define FILES_OLD_KIND 1
#define FILES_NEW_KIND 2

typedef struct {
  int		N_kill; /* These are Zeroed by print, Incremented by put */
  kill_type	*kill;	/*  Set by scan, Decremented by get 		 */
  int		kill_size;

  int		N_old;	
  old_type	*old;
  int		old_size;

  int		N_new;
  new_type	*new;
  int		new_size;

  char		*filename;
  FILE		*fp;
  int		mode;

  int		(*get_kill)();
  old_type	*(*get_old)();
  new_type	*(*get_new)();
  void		(*put_kill)();
  void		(*put_old)();
  void		(*put_new)(/* fts_type *, new_type * */);
  int		(*scan)();
  void		(*print)();
  void		(*close)();
} fts_type;

extern fts_type *fts_open_read();
extern fts_type *fts_open_write();

               /********************************\
----------------------------------------------------------------
  (.est) files
----------------------------------------------------------------
               \********************************/

typedef struct { int id; float a; } alpha_type;

typedef struct {
  float		x, y, zb;
  float		q0, q1, q2, q3;
  float		b;
  float		xp, yp;
  int		N;
  alpha_type	*alphas;
  int		alphas_size;

  char		*filename;
  FILE		*fp;
  int		mode;

  alpha_type	*(*get_alpha)();
  void		(*put_alpha)(/* est_type *, alpha_type * */);
  int		(*scan)();
  void		(*print)();
  void		(*close)();
} est_type;

extern est_type	*est_open_read();
extern est_type	*est_open_write();

               /********************************\
----------------------------------------------------------------
  (.sta) files
----------------------------------------------------------------
               \********************************/

typedef struct { int id; float a, Va, x, y; } depth_type;

typedef struct {
  int		iter;
  float		b, Vb;
  float		xp, yp, Vxp, Vyp;
  float		dz;
  int		N;
  depth_type	*depths;
  int		depths_size;

  char		*filename;
  FILE		*fp;
  int		mode;

  depth_type	*(*get_depth)();
  void		(*put_depth)();
  int		(*scan)();
  void		(*print)();
  void		(*close)();
} sta_type;

extern sta_type	*sta_open_read();
extern sta_type	*sta_open_write();

#endif /* _FILES_H_ */
