/*
###########################################################################
# 
# Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
# rights reserved.
# 
# Developed  by Ali Azarbayejani and Alex Pentland,  and Martin  Friedmann
# at the Media Laboratory, MIT, Cambridge, Massachusetts.
# 
# Permission to use, copy,  or modify this software and  its documentation
# for  educational  and  research purposes only and without fee  is hereby
# granted, provided  that this copyright notice and the original authors's
# names appear  on all copies and supporting documentation.  If individual
# files are  separated from  this  distribution directory  structure, this
# copyright notice must be included.  For any other uses of this software,
# in original or  modified form, including but not limited to distribution
# in whole or in  part, specific  prior permission  must be  obtained from
# MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
#   basis  of  a commercial  software  or  hardware product  without first
# obtaining appropriate licenses  from MIT.  MIT. makes no representations
# about the suitability of this  software for any purpose.  It is provided
# "as is" without express or implied warranty.
# 
###########################################################################

  fts.c
  26 May 1993

  AUTHOR:
    Ali J. Azarbayejani

  CONTENTS:
    fts_error()
    fts_warning()
    fts_list_init()
    fts_list_expand()
    fts_list_check()
    fts_list_input()
    fts_get_kill()
    fts_get_old()
    fts_get_new()
    fts_put_kill()
    fts_put_old()
    fts_put_new()
    fts_scan()
    fts_print()
    fts_close()
    fts_open_read()
    fts_open_write()

  DESCRIPTION:
    Feature data and file format.

*/

#include <stdio.h>
#include <stdlib.h>
#include "files.h"

               /********************************\
----------------------------------------------------------------
   Utility functions
----------------------------------------------------------------
               \********************************/

static void
  fts_error(char *s)
{
  fprintf(stderr, "fts ERROR: %s\n", s);
  exit(0);
}

static void
  fts_warning(char *s)
{
  fprintf(stderr, "fts WARNING: %s\n", s);
}

               /********************************\
----------------------------------------------------------------
   List handlers.
   These can be taken out and used in general for maintaining
    lists of things.
----------------------------------------------------------------
               \********************************/

static void *
  fts_list_init(size_t obj_size, int *n_total_p)
{
  void * ptr = malloc(FILES_LIST_INIT * obj_size);

  if(NULL == ptr)
    fts_error("fts_list_init: failed to malloc");

  *n_total_p = FILES_LIST_INIT;
  return ptr;
}

static void *
  fts_list_expand(size_t obj_size, void *list, int n_used, int* n_total_p)
{
  void * ptr;

  if(obj_size < 0 || NULL == list || n_used < 0 || *n_total_p <= 0
     || n_used > *n_total_p)
    fts_error("fts_list_expand: stupid args");

  ptr = malloc( (*n_total_p + FILES_LIST_INC) * obj_size );

  if(NULL == ptr)
    fts_error("fts_list_expand: malloc failed");

  bcopy(list, ptr, n_used * obj_size);
  free(list);

  *n_total_p += FILES_LIST_INC;
  return ptr;
}

/* Outputs valid "list" and "*n_total_p" */
static void *
  fts_list_check(void *list, int n_used, int *n_total_p, size_t obj_size)
{
  if(list && n_used < *n_total_p)
    return list;

  else if(NULL == list && 0 == n_used && 0 == *n_total_p && obj_size > 0)
    return fts_list_init(obj_size, n_total_p);

  else if(list && n_used == *n_total_p && obj_size > 0)
    return fts_list_expand(obj_size, list, n_used, n_total_p);

  else
    fts_error("fts_list_check: uh-oh, error in args?");
}

static void
  fts_list_input(void *in, void **l_p, int *nu_p, int *nt_p, size_t obj_size)
{
  void *ptr;

  *l_p = fts_list_check(*l_p, *nu_p, nt_p, obj_size);
  
  ptr = (char *)(*l_p) + (*nu_p) * obj_size;
  bcopy(in, ptr, obj_size);
  (*nu_p)++;
}

               /********************************\
----------------------------------------------------------------
   External functions.
----------------------------------------------------------------
               \********************************/

static int
  fts_get_kill(fts_type *F)
{
  if(FILES_OPEN_READ == F->mode) {
    if(F->N_kill > 0)
      return( *((int *)F->kill + --F->N_kill));
    else
      return EOF;
  }
  else
    fts_warning("fts_get_kill: file not open for read");
}

static old_type *
  fts_get_old(fts_type *F)
{
  if(FILES_OPEN_READ == F->mode) {
    if(F->N_old > 0)
      return( F->old + --F->N_old );
    else
      return ((void *)EOF);
  }
  else
    fts_warning("fts_get_old: file not open for read");
}

static new_type *
  fts_get_new(fts_type *F)
{
  if(FILES_OPEN_READ == F->mode) {
    if(F->N_new > 0)
      return( F->new + --F->N_new );
    else
      return ((void *)EOF);
  }
  else
    fts_warning("fts_get_new: file not open for read");    
}

static void
  fts_put_kill(fts_type *F, int id)
{
  if(FILES_OPEN_WRITE == F->mode)
    fts_list_input((void *)&id, (void **)&F->kill,
		   &F->N_kill, &F->kill_size, sizeof(kill_type));
  else 
    fts_warning("fts_put_kill: file not open for write");
}

static void
  fts_put_old(fts_type *F, old_type *op)
{
  if(FILES_OPEN_WRITE == F->mode)
    fts_list_input((void *)op, (void **)&F->old,
		   &F->N_old, &F->old_size, sizeof(old_type));
  else 
    fts_warning("fts_put_old: file not open for write");
}

static void
  fts_put_new(fts_type *F, new_type *np)
{
  if(FILES_OPEN_WRITE == F->mode)
    fts_list_input((void *)np, (void **)&F->new,
		   &F->N_new, &F->new_size, sizeof(new_type));
  else
    fts_warning("fts_put_new: file not open for write");
}

static int
  fts_scan(fts_type *F)
{
  int count, nconv, total = 0;
  kill_type kt;
  old_type or;
  new_type nr;

  if(F->mode != FILES_OPEN_READ) {
    fts_warning("fts_scan: File not open for read");
    return EOF;
  }

  F->N_kill = F->N_old = F->N_new = 0;

  nconv = fscanf(F->fp, "%*s%d", & count);
  if(EOF == nconv)
    return EOF;
  else if(1 != nconv)
    fts_error("fts_scan: Read error kill");
  total += count;
  while(count--) {
    if(1 != fscanf(F->fp, "%d", &kt))
      fts_error("fts_scan: Read error kill record");
    fts_list_input((void *)&kt, (void **)&(F->kill),
		   &(F->N_kill), &(F->kill_size), sizeof(kill_type));
  }

  if(1 != fscanf(F->fp, "%*s%d", &count))
    fts_error("fts_scan: Read error old");
  total += count;
  while(count--) {
    nconv = fscanf(F->fp, "%d%g%g%g%g",
		   &or.id, &or.x, &or.y, &or.Vx, &or.Vy);
    if( 5 != nconv ) {
      fprintf( stderr, "fts_scan: old_record: read %d [5] values (%d, %f, %f)\n",
	      nconv, or.id, or.x, or.y );
      fts_error("fts_scan: Read error old record");
    }
    fts_list_input((void *)&or, (void **)&F->old,
		   &F->N_old, &F->old_size, sizeof(old_type));
  }

  if(1 != fscanf(F->fp, "%*s%d", &count))
    fts_error("fts_scan: Read error new");
  total += count;
  while(count--) {
    if(3 != fscanf(F->fp, "%d%g%g",
		   &nr.id, &nr.x, &nr.y))
      fts_error("fts_scan: Read error new record");
    fts_list_input((void *)&nr, (void **)&F->new,
		   &F->N_new, &F->new_size, sizeof(new_type));
  }
  return total;
}

static void
  fts_print(fts_type *F)
{
  int kt;
  old_type *op;
  new_type *np;

  if(NULL != F->fp && FILES_OPEN_WRITE == F->mode
     && F->N_kill >=0 && F->N_old >= 0 && F->N_new >= 0) {

    fprintf(F->fp, "KILL %d\n", F->N_kill);
    while(F->N_kill) {
      kt = *(F->kill + --F->N_kill);
      fprintf(F->fp, "  %d\n", kt);
    }

    fprintf(F->fp, "OLD %d\n", F->N_old);
    while(F->N_old) {
      op = F->old + --F->N_old;
      fprintf(F->fp, "  %d %.10g %.10g %.10g %.10g\n",
	      op->id, op->x, op->y, op->Vx, op->Vy);
    }

    fprintf(F->fp, "NEW %d\n", F->N_new);
    while(F->N_new) {
      np = F->new + --F->N_new;
      fprintf(F->fp, " %d %.10g %.10g\n", np->id, np->x, np->y);
    }
    
    fprintf(F->fp, "\n");
  }
  else
    fts_error("fts_print: bad args");
}

static void
  fts_close(fts_type *F)
{
  if(FILES_CLOSED == F->mode) {
    fts_warning("fts_close: file aint open");
    return;
  }
  F->filename = NULL;
  fclose(F->fp);
  F->fp = NULL;
  F->mode = FILES_CLOSED;
}

fts_type *
  fts_open_read(char *name)
{
  FILE *fp;
  register fts_type *F = NULL;

  fp = (NULL == name) ? stdin : fopen(name, "r");
#ifdef VX
  if(1) {
#else
  if(NULL != fp) {
#endif
    F = (fts_type *) malloc(sizeof(fts_type));
    if(NULL != F) {

      F->N_kill = 0;
      F->kill = NULL;
      F->kill_size = 0;

      F->N_old = 0;
      F->old = NULL;
      F->old_size = 0;

      F->N_new = 0;
      F->new = NULL;
      F->new_size = 0;

      F->filename = name;
      F->fp = fp;
      F->mode = FILES_OPEN_READ;

      F->scan = fts_scan;
      F->print = fts_print;
      F->get_kill = fts_get_kill;
      F->get_old = fts_get_old;
      F->get_new = fts_get_new;
      F->put_kill = fts_put_kill;
      F->put_old = fts_put_old;
      F->put_new = fts_put_new;
      F->close = fts_close;
    }
    else
      fclose(fp);
  }
  return F;
}

fts_type *
  fts_open_write(char *name)
{
  FILE *fp;
  register fts_type *F = NULL;

  fp = (NULL == name) ? stdout : fopen(name, "w");
  if(NULL != fp) {
    F = (fts_type *) malloc(sizeof(fts_type));

    if(NULL != F) {

      F->N_kill = 0;
      F->kill = NULL;
      F->kill_size = 0;

      F->N_old = 0;
      F->old = NULL;
      F->old_size = 0;

      F->N_new = 0;
      F->new = NULL;
      F->new_size = 0;

      F->filename = name;
      F->fp = fp;
      F->mode = FILES_OPEN_WRITE;

      F->scan = fts_scan;
      F->print = fts_print;
      F->get_kill = fts_get_kill;
      F->get_old = fts_get_old;
      F->get_new = fts_get_new;
      F->put_kill = fts_put_kill;
      F->put_old = fts_put_old;
      F->put_new = fts_put_new;
      F->close = fts_close;
    }
    else
      fclose(fp);
  }
  return F;
}
