/*
###########################################################################
# 
# Copyright 1995  by  the  Massachusetts Institute  of Technology.   All  
# rights reserved.
# 
# Developed  by Ali Azarbayejani and Alex Pentland,  and Martin  Friedmann
# at the Media Laboratory, MIT, Cambridge, Massachusetts.
# 
# Permission to use, copy,  or modify this software and  its documentation
# for  educational  and  research purposes only and without fee  is hereby
# granted, provided  that this copyright notice and the original authors's
# names appear  on all copies and supporting documentation.  If individual
# files are  separated from  this  distribution directory  structure, this
# copyright notice must be included.  For any other uses of this software,
# in original or  modified form, including but not limited to distribution
# in whole or in  part, specific  prior permission  must be  obtained from
# MIT.  These programs shall not  be  used, rewritten, or  adapted as  the
#   basis  of  a commercial  software  or  hardware product  without first
# obtaining appropriate licenses  from MIT.  MIT. makes no representations
# about the suitability of this  software for any purpose.  It is provided
# "as is" without express or implied warranty.
# 
###########################################################################

  sqtosta.c
    2 June 1993

  AUTHOR:
    Ali J. Azarbayejani 2 June 1993 (originated)

  CONTENTS:
    ParseCmdLine()
    shift_points_positiveZ()
    main()

  DESCRIPTION:
    (.sq) --> (.sta)

*/

#include <stdio.h>
#include "files.h"

static struct {
  int	verbose;
}
params = { 0 };

static void
  ParseCmdLine(int argc, char **argv)
{
  int		i;
  char		usage[1024], io[128], *arg;

  sprintf(usage, "%s", "[-v]");
  sprintf(io,"\n\t%s", "< (.sq) > (.sta)");

  for(i=1; i<argc; i++) {
    if(!strcmp(argv[i], "-v"))
      params.verbose = 1;
    else {
      fprintf(stderr,
	      "usage: %s %s %s\n",
	      argv[0], usage, io);
      exit(1);
    }
  }
}

float
  shift_points_positiveZ(float *pts, int npts)
{
  float minZ = 0.;
  int count;

  for(count = 0; count < npts; count++)
    if( pts[3*count + 2] < minZ )
      minZ = pts[3*count + 2];

  for(count = 0; count < npts; count++)
    pts[3*count + 2] -= minZ;

  return minZ;
}

main(int argc, char **argv)
{
  sta_type	*Shape;
  int		npts, count;
  float		*pts, *ptr;
  depth_type	dt;
  float         beta;
  
  scanf("%f", &beta);

  scanf("%d", &npts);
  pts = (float *) malloc( 3*sizeof(float)*npts );
  if(!pts) {
    fprintf(stderr, "sqtosta: failed to malloc\n");
    exit(0);
  }
  count = 3*npts;
  ptr = pts;
  while(count--)
    scanf("%g", ptr++);

  Shape = sta_open_write(NULL);

/* CCC  new, commented out, we want always offset 0
  Shape->dz = shift_points_positiveZ(pts, npts);
*/

  for(count = 0; count < npts; count++) {
    dt.id = count;
    dt.a = pts[3*count + 2];
    dt.Va = 0.;
    dt.x = pts[3*count] / (1. + beta*dt.a) + Shape->xp;
    dt.y = pts[3*count + 1] / (1. + beta*dt.a) + Shape->yp;
    Shape->put_depth(Shape, &dt);
    Shape->b = beta;
  }

  Shape->print(Shape);

}
